%define _disable_ld_no_undefined 0

%define dirrname %{version}.src

%define gcc_version %(gcc -dumpversion)

%bcond_with apidox
%bcond_without clang

%define major 3.7
%define minor 1

%define fversion %major.%minor
%define version %major.%minor

%define libname %mklibname %name %major
%define libname_devel %mklibname -d %name

%if %{with clang}
%define clang_major %fversion
%define clang_libname %mklibname clang %clang_major
%define clang_libname_devel %mklibname -d clang
%endif


Name:		llvm
Version:	%version
Release:	%mkrel 2
Summary:	Low Level Virtual Machine (LLVM)
License:	NCSA
Group:		Development/Other
URL:		http://llvm.org
Source0:	http://llvm.org/releases/%{version}/llvm-%{dirrname}.tar.xz
Source1:	http://llvm.org/releases/%{version}/cfe-%{version}.src.tar.xz
Source2:	http://llvm.org/releases/%{version}/compiler-rt-%{dirrname}.tar.xz
Patch0:		clang-soname.patch
Obsoletes:	llvm-ocaml
Requires:	libstdc++-devel
BuildRequires:	gcc
BuildRequires:	bison
BuildRequires:	groff
BuildRequires:	chrpath
BuildRequires:	ocaml
BuildRequires:	tcl
BuildRequires: python-sphinx
%if %{with apidox}
BuildRequires:	doxygen
%endif
BuildRequires:	flex
BuildRequires:	sed
BuildRequires:	graphviz
BuildRequires:	libstdc++-devel
BuildRequires:	libtool
BuildRequires:	zip
BuildRequires:	libffi-devel
BuildRequires:	chrpath

%description
LLVM is a robust system, particularly well suited for developing new mid-level
language-independent analyses and optimizations of all sorts, including those
that require  extensive interprocedural analysis. LLVM is also a great target
for front-end development for conventional or research programming languages,
including those which require compile-time, link-time, or run-time optimization 
for effective implementation, proper tail calls or garbage collection. 


%package -n llvm-devel
Summary:	Development files for LLVM
Group:		Development/Libraries
Provides:	llvm-devel = %version-%release
Requires:	llvm

%description -n llvm-devel
This package contains the development files for LLVM;

%package doc
Summary:	Documentation for LLVM
Group:		Documentation
Requires:	%{name} = %{version}
BuildArch:	noarch

%description doc
Documentation for the LLVM compiler infrastructure.

%if %{with apidox}

%package apidoc
Summary:	API Documentation for LLVM
Group:		Documentation
Requires:	%{name} = %{version}
BuildArch:	noarch

%description apidoc
Documentation for the LLVM compiler infrastructure.

%endif

%if %{with clang}

%package -n clang
Summary:	A C language family front-end for LLVM
License:	NCSA
Group:		Development/Other
Requires:	llvm
Requires:	gcc

%description -n clang
clang: noun
    1. A loud, resonant, metallic sound.
    2. The strident call of a crane or goose.
    3. C-language family front-end toolkit.

The goal of the Clang project is to create a new C, C++, Objective C
and Objective C++ front-end for the LLVM compiler. Its tools are built
as libraries and designed to be loosely-coupled and extensible.

%package -n clang-devel
Summary:	Development files for clang
Group:		Development/Libraries
Requires:	clang
Provides:	clang-devel = %version-%release

%description -n clang-devel
This package contains header files and libraries needed for using
libclang.

%package -n clang-analyzer
Summary:	A source code analysis framework
License:	NCSA
Group:		Development/Other
Requires:	clang
Requires:	python

%description -n clang-analyzer
The Clang Static Analyzer consists of both a source code analysis
framework and a standalone tool that finds bugs in C and Objective-C
programs. The standalone tool is invoked from the command-line, and is
intended to run in tandem with a build of a project or code base.

%package -n clang-doc
Summary:	Documentation for Clang
Group:		Documentation
BuildArch:	noarch
Requires:	clang

%description -n clang-doc
Documentation for the Clang compiler front-end.

%endif

%prep

%setup -qn %{name}-%{version}.src %{?with_clang:-a1 -a2}
rm -rf tools/clang

%if %{with clang}
mv cfe-%{fversion}.src tools/clang
cd tools/clang
%patch0 -p1
cd -

mv compiler-rt-%{fversion}.src projects/compiler-rt
%endif
sed -ri "/ifeq.*CompilerTargetArch/s#i386#i586#g" projects/compiler-rt/make/platform/clang_linux.mk

%build

export CC=gcc
export CXX=g++

cd autoconf
aclocal --force -I m4
autoconf --force -o ../configure configure.ac
cd ..
autoheader -I autoconf -I autoconf/m4 autoconf/configure.ac

mkdir build
pushd build
export CFLAGS="%{optflags} -DLLDB_DISABLE_PYTHON -DHAVE_PROCESS_VM_READV"
export CXXFLAGS="%{optflags} -DLLDB_DISABLE_PYTHON -DHAVE_PROCESS_VM_READV"
ln -s ../configure .
%configure2_5x \
	--libdir=%{_libdir}/%{name} \
	--datadir=%{_datadir}/%{name} \
	--enable-shared \
	--enable-jit \
	--enable-libffi \
	--enable-optimized \
%ifarch %ix86 x86-64
	--enable-targets="x86,x86_64" \
%else
	--enable-targets=host-only \
%endif
	--disable-expensive-checks \
	--enable-debug-runtime \
	--disable-assertions \
	--enable-threads \
	--enable-experimental-targets=AMDGPU \
	--disable-bindings \
%if %{with apidox}
	--enable-doxygen
%endif

# FIXME file this
# configure does not properly specify libdir
sed -i 's|(PROJ_prefix)/lib|(PROJ_prefix)/%{_lib}/%{name}|g' Makefile.config

# FIXME upstream need to fix this
# llvm-config.cpp hardcodes lib in it
sed -i 's|ActiveLibDir = ActivePrefix + "/lib"|ActiveLibDir = ActivePrefix + "/%{_lib}/%{name}"|g' ../tools/llvm-config/llvm-config.cpp

%define enable_rtti export REQUIRES_RTTI=1
%{enable_rtti}
%make
# Make docs

pushd docs
# As of 3.2, LLVM doesn't build man pages by itself. Aargh!
# Make doxygen docs
%if %{with apidox}
doxygen doxygen.cfg
%endif
popd

%install
pushd build
%{enable_rtti}

%makeinstall_std \
	KEEP_SYMBOLS=1 \
	PROJ_docsdir=%{_docdir}/%{name} \
	PROJ_etcdir=%{_sysconfdir}/%{name} \
	PROJ_libdir=%{_libdir}/%{name}
popd

rm -rf %buildroot%_bindir/.dir

file %{buildroot}/%{_bindir}/* | awk -F: '$2~/ELF/{print $1}' | xargs -r chrpath -d
file %{buildroot}/%{_libdir}/llvm/*.so | awk -F: '$2~/ELF/{print $1}' | xargs -r chrpath -d

mv %{buildroot}%{_libdir}/llvm/libLLVM-%major.so %{buildroot}%{_libdir}
ln -s libLLVM-%major.so %{buildroot}%{_libdir}/libLLVM.so


for staticlib in %{buildroot}%{_libdir}/llvm/libLLVM*.a; do
	sharedlib="${staticlib%.a}.so"
	[ -e "$sharedlib" ] && exit 1
	ln -s ../libLLVM.so "$sharedlib"
done

# Install man pages
#cp docs/_build/man/*.1 %{buildroot}%{_mandir}/man1

%if %with clang
mv %{buildroot}%{_libdir}/llvm/libclang.so %{buildroot}%{_libdir}/libclang-%version.so
ln -s libclang-%clang_major.so %{buildroot}%{_libdir}/libclang.so
ln -s ../libclang.so %{buildroot}%{_libdir}/llvm/libclang.so

%endif

strip --strip-debug %{buildroot}%{_libdir}/llvm/*.a

%if %{with clang}

mkdir -p %{buildroot}%{_libdir}/clang-analyzer

# create launchers
for f in scan-{build,view}; do
  ln -s %{_libdir}/clang-analyzer/$f/$f %{buildroot}%{_bindir}/$f
done

(cd tools/clang/tools && cp -pr scan-{build,view} \
 %{buildroot}%{_libdir}/clang-analyzer/)

# scan-build tries to find clang relatively to its real path
ln -s %{_bindir}/clang %{buildroot}%{_libdir}/clang-analyzer/scan-build/

# And prepare Clang documentation
#
rm -rf clang-docs
mkdir clang-docs
for f in LICENSE.TXT NOTES.txt README.txt; do # TODO.txt; do
  ln tools/clang/$f clang-docs/
done
rm -rf clang-docs-full
cp -al tools/clang/docs clang-docs-full
rm -rf clang-docs-full/{doxygen*,Makefile*,*.graffle,tools}

%endif

# Get rid of erroneously installed example files.
rm %{buildroot}%{_libdir}/%{name}/LLVMHello.so

%files
%doc LICENSE.TXT
%{_bindir}/FileCheck
%{_bindir}/count
%{_bindir}/llvm-profdata
%{_bindir}/not
%{_bindir}/bugpoint
%{_bindir}/llc
%{_bindir}/lli
%{_bindir}/lli-child-target
%{_bindir}/obj2yaml
%{_bindir}/opt
%{_bindir}/llvm-ar
%{_bindir}/llvm-as
%{_bindir}/llvm-bcanalyzer
%{_bindir}/llvm-cxxdump
%{_bindir}/llvm-diff
%{_bindir}/llvm-dsymutil
%{_bindir}/llvm-cov
%{_bindir}/llvm-dis
%{_bindir}/llvm-dwarfdump
%{_bindir}/llvm-extract
%{_bindir}/llvm-link
%{_bindir}/llvm-mc
%{_bindir}/llvm-mcmarkup
%{_bindir}/llvm-nm
%{_bindir}/llvm-objdump
%{_bindir}/llvm-pdbdump
%{_bindir}/llvm-ranlib
%{_bindir}/llvm-readobj
%{_bindir}/llvm-rtdyld
%{_bindir}/llvm-size
%{_bindir}/llvm-stress
%{_bindir}/llvm-symbolizer
%{_bindir}/llvm-tblgen
%{_bindir}/macho-dump
%{_bindir}/verify-uselistorder
%{_bindir}/yaml2obj
%{_libdir}/libLLVM-%major.so

%files devel
%{_bindir}/%{name}-config
%{_libdir}/libLLVM.so
%{_libdir}/libLLVM-%{version}.so
%{_includedir}/%{name}
%{_includedir}/%{name}-c
%dir %{_libdir}/%{name}
%{_libdir}/%{name}/BugpointPasses.so
%{_libdir}/%{name}/libLLVM*.a
%{_libdir}/%{name}/libLLVM*.so
%{_libdir}/%{name}/libLTO.a
%{_libdir}/%{name}/libLTO.so
%dir %{_datadir}/llvm
%{_datadir}/llvm/cmake/*.cmake

%doc CREDITS.TXT LICENSE.TXT README.txt
#%doc docs/*.css
%doc docs/*.html
%doc docs/_static
%doc docs/tutorial
%doc docs/CommandGuide
%doc examples

%if %{with apidoc}
%files apidoc
%doc docs/doxygen/html/*
%endif

%if %{with clang}

%files -n clang
%{_bindir}/clang*
%{_bindir}/c-index-test
%{_prefix}/lib/clang

%{_libdir}/libclang-%clang_major.so

%files -n clang-devel
%{_includedir}/clang
%{_includedir}/clang-c
%{_libdir}/libclang.so
%dir %{_libdir}/%{name}
%{_libdir}/%{name}/libclang*.a
%{_libdir}/%{name}/libclang*.so

%files -n clang-analyzer
%{_bindir}/scan-build
%{_bindir}/scan-view
%{_libdir}/clang-analyzer

%files -n clang-doc
#%{_docdir}/%{name}/html/clang/*
%endif


%changelog
* Wed Jan 06 2016 bb <bb> 3.7.1-2pclos2016
- fix build

* Tue Jan 05 2016 bb <bb> 3.7.1-1pclos2016
- update

* Wed Sep 23 2015 bb <bb> 3.7.0-1pclos2015
- update

* Sun Jul 26 2015 bb <bb> 3.6.2-1pclos2015
- update

